/*********************************************************************************/
/* $Id: Config.c,v 1.40 2002/05/27 17:25:15 Anoosh Naderi Exp $ 		 */
/*										 */
/* Copyright (c) 2002, Analog Devices Inc., All Rights Reserved			 */
/*										 */
/* Config.c									 */
/*										 */
/* Various routines that implement a minimal ini-style configuration file system */
/*										 */
/*********************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "Zaps.h"
#include "Stuff.h"
#include "Konst.h"
#include "Options.h"

/*********************************************************************************/
/* Local prototypes								 */
/*********************************************************************************/

static Boolean ConfigGetEntryText(char *pOption, char *pValue, FILE *fp);
static Boolean ConfigGetEntryInt(char *pOption, UInt32 *pValue, FILE *fp);
static Boolean ConfigOptionExists(char *srcBuffer, char *Option);
static Boolean ConfigExtractValue(char *srcBuffer, char *Destination);
static Boolean ConfigTrim(char *str);
static UInt8 htoi(UInt8 h);

/*********************************************************************************/
/* ConfigGetEntryText								 */
/*										 */ 
/* Given a specified option, returns its value if it exists in the		 */
/* config file.									 */
/*********************************************************************************/
static Boolean ConfigGetEntryText(char *pOption, char *pValue, FILE *fp)
{
    Boolean rv = false;
    char buffer[256];

    /* First we need to see if this key already exists*/
    fseek(fp, 0, SEEK_SET);

    /* Search the entire file*/
    while ( feof(fp) == 0 )
    {
       /* Read a line from the file*/
       if ( fgets(buffer, 256, fp) )
       {
          /* Does this line have our option*/
          if ( ConfigOptionExists(buffer, pOption) )
          {
             if ( ConfigExtractValue(buffer, pValue) )
             {
                ConfigTrim(pValue);
                rv = true;
             }
             break;
          }
       }
    }
    return ( rv );
}

/*********************************************************************************/
/* ConfigOptionExists								 */
/*										 */
/* Returns TRUE if the specified Option exists in the config file		 */
/*********************************************************************************/
static Boolean ConfigOptionExists(char *srcBuffer, char *Option)
{
    Boolean rv = false;
    char buffer[256];
    char *pChar;

    /* We copy the buffer because wo modify it*/
    strcpy(buffer, srcBuffer);

    /* Look for the terminator*/
    pChar = strtok(buffer, "=");
    /* Is it there ?*/
    if ( pChar )
    {
       /* Copy the buffer*/
       strcpy(buffer, pChar);
       
       /* Remove Leading and trailing whitespace*/
       ConfigTrim(buffer);
        
       /* Yep, get the next piece*/
       if ( strcmp(buffer, Option) == 0 )
       {
          rv = true;
       }
    }
    return ( rv );
}

/*********************************************************************************/
/* ConfigExtractValue								 */
/*										 */
/* Given an 'option=value' item in a string, extracts the value portion		 */
/* and stores it in the Destination param					 */
/*********************************************************************************/
static Boolean ConfigExtractValue(char *srcBuffer, char *Destination)
{
    Boolean rv = false;
    char buffer[256];
    char *pChar;

    /* We copy the buffer because wo modify it*/
    strcpy(buffer, srcBuffer);

    /* Look for the terminator*/
    pChar = strtok(buffer, "=");
    /* Is it there ?*/
    if ( pChar )
    {
       /* Yep, get the next piece*/
       pChar = strtok(NULL, "\n\r");
       /* Is it there*/
       if ( pChar )
       {
          /* Copy the string*/
          strcpy(Destination, pChar);
          /* And trim*/
          ConfigTrim(Destination);
          /* Set rv*/
          rv = true;
       }
    }
    return ( rv );
}

/*********************************************************************************/
/* ConfigTrim									 */
/*										 */
/* Trims excess whitespace from the beginning and end of the specified string    */
/*********************************************************************************/
static Boolean ConfigTrim(char *str)
{
    Boolean rv = true;
    char *pChar = str + strlen(str) - 1;
    char buffer[256];
    int count = 0;
    
    /* remove trailing white space first*/
    while ( isspace(*pChar) )
    {
       *pChar-- = '\0';
    }
    
    /* remove leading whitespace*/
    pChar = str;
    while ( isspace(*pChar) )
    {
       count ++;
       pChar ++;
    }
    
    /* Copy over the source string without leading space*/
    strcpy(buffer, str + count);
    /* and then copy it back into our buffer*/
    strcpy(str, buffer); 

    return ( rv );
}

/*********************************************************************************/
/* ConfigGetOptions								 */
/*										 */
/* Reads the array of supplied DriverOptions values from the configuration file  */ 
/*********************************************************************************/
void ConfigGetOptions( void )
{
    int loop = 0;
    DriverOption *pOptions;
    FILE *fp;

    /* Open it*/
    if ((fp = fopen(kCONFIG_FILENAME, "rt")))
    {
       pOptions = drvOptions;

       ZAP(DEBUG_CONFIG, DEBUG_INFO, ("Opened it\n"));

       /* For each DriverOption*/
       for ( loop = 0 ; loop < NUM_DRV_OPTIONS ; loop ++ )
       {
           ZAP(DEBUG_CONFIG, DEBUG_INFO, ("Trying to read %s\n",pOptions[loop].Name ));

           /* Try and read the data */               
           if ( ! ConfigGetEntryInt(pOptions[loop].Name, &pOptions[loop].Value, fp) )
           {
              pOptions[loop].Value = 0xF9F8F7F6;
              ZAP(DEBUG_CONFIG, DEBUG_INFO, ("config file could not read %s\n", pOptions[loop].Name));
           }
           else
           {
              ZAP(DEBUG_CONFIG, DEBUG_INFO,
                  ("Option %s assigned value %#x\n", pOptions[loop].Name, pOptions[loop].Value));
           }
       }
       fclose(fp);
    }
    else
       ZAP(DEBUG_CONFIG, DEBUG_ERRORS, (kTAB"Config file %s does not exist\n", kCONFIG_FILENAME));
}   

/*********************************************************************************/
/* ConfigGetEntryInt								 */
/*										 */
/* Reads a value for the specified option from the config file			 */
/*********************************************************************************/
static Boolean ConfigGetEntryInt(char *pOption, UInt32 *pValue, FILE *fp)
{
    char buffer[256];
    Boolean rv = false;
   
    int i;
    char b[4];
    char TmpBuffer[20];
    int TmpLen=0;
    int ii=0;
    int jj=0;
    int kk=0;

    for (ii=0;ii<20;ii++)
        TmpBuffer[ii]='0';    

    /* Use the text code*/
    if ( ConfigGetEntryText(pOption, buffer, fp) )
    {
       TmpLen=strlen(buffer);
       for (ii=0;ii<8-TmpLen;ii++)
       {
           jj++;
           TmpBuffer[ii]='0';
       }
       kk=0;
       for (ii=jj;ii<8;ii++)
       {
           TmpBuffer[ii]=buffer[kk];
           kk++;
       }

       for (i=0; i<4; i++)
       {
           b[3-i] = (htoi(TmpBuffer[i*2])<<4) + (htoi(TmpBuffer[i*2+1]));
       }

       *pValue = *((UInt32 *)b);

       /* set rv*/
       rv = true;
    }
        
    return ( rv );      
}

/*********************************************************************************/
/* htoi										 */
/*										 */
/* Converts a hex char to an integer (my own wacky implementation).		 */
/* I don't know why this isn't part of a standard library or something,		 */
/* it seems like this is a quite-frequently-rewritten function.			 */
/*********************************************************************************/
static UInt8 htoi(UInt8 h)
{
    switch (toupper(h))
    {
       case '0':
       case '1':
       case '2':
       case '3':
       case '4':
       case '5':
       case '6':
       case '7':
       case '8':
       case '9':
          return h-'0';
          break;
       case 'A':
       case 'B':
       case 'C':
       case 'D':
       case 'E':
       case 'F':
          return h-'A'+10;
          break;
    }

    /* If we get here ... the character was invalid*/
    return 0;
}

/***********************************************************************
$Log: Config.c,v $
Revision 1.40  2002/05/27 17:25:15  Anoosh Naderi
Clean up the code

Revision 1.3  2002/01/23 17:25:15  chris.edgington
Detabbed.

Revision 1.2  2002/01/17 21:30:35  chris.edgington
Moved include order of Zaps.h to keep ZAP from being redefined.

Revision 1.1  2002/01/17 20:37:54  chris.edgington
First version.
***********************************************************************/
